@@ -21,8 +21,7 @@ from ..scheduler import ConfigError, TaskComplete
 from ..secure import aes
 from ..secure.ecdhe import new_key_pair, new_shared_key
 from ..secure.shamir import SecretShare
-from .contractor import (CheckinEvent, FinishTaskEvent, ResetRoundEvent,
-                         UploadTrainingResultsEvent)
+from .contractor import CheckinEvent, FinishTaskEvent, ResetRoundEvent
 from .fed_avg import AggregationError, FedAvgScheduler, ResetRound
 from .secure_contractor import (AdvertiseKeysEvent, AdvertiseKeysResponseEvent,
                                 DistributeUser1ListEvent,
@@ -356,7 +355,6 @@ class SecureFedAvgScheduler(FedAvgScheduler):
     def _wait_for_calculation(self) -> Tuple[Dict[str, torch.Tensor], int]:
         """Wait for every calculator finish its task or timeout."""
         self._users_3.clear()
-        user_2_num = self._get_user_2_num()
         accum_result = self.state_dict()
         for _key, _param in accum_result.items():
             if isinstance(_param, torch.Tensor):
@@ -365,24 +363,18 @@ class SecureFedAvgScheduler(FedAvgScheduler):
                 logger.warn(f'not Tensor value: {_key=}; {_param=}; {type(_param)=}')
 
         self.push_log('Waiting for training results ...')
-        for _event in self.contractor.contract_events(timeout=self.calculation_timeout):
-            if isinstance(_event, UploadTrainingResultsEvent):
-                if self.id not in _event.target:
-                    continue
-                training_result = self.data_channel.receive_stream(apply_event=_event,
-                                                                   receiver=self.id)
-                buffer = io.BytesIO(training_result)
-                _new_state_dict = torch.load(buffer)
-                for _key in accum_result.keys():
-                    accum_result[_key].add_(_new_state_dict[_key])
-                self._users_3.append(_event.source)
-                self.push_log(f'Received calculation results from ID: {_event.source}')
-
-                if len(self._users_3) >= user_2_num:
-                    return accum_result, len(self._users_3)
-
-            elif isinstance(_event, CheckinEvent):
-                self._handle_check_in(_event)
+        training_results = self.data_channel.batch_receive_stream(
+            receiver=self.id,
+            source_list=[_user.source_id for _user in self._users_2],
+            timeout=self.calculation_timeout
+        )
+        for _source, _result in training_results.items():
+            buffer = io.BytesIO(_result)
+            _new_state_dict = torch.load(buffer)
+            for _key in accum_result.keys():
+                accum_result[_key].add_(_new_state_dict[_key])
+            self._users_3.append(_source)
+            self.push_log(f'Received calculation results from ID: {_source}')
 
         if len(self._users_3) < self._t:
             self.push_log('Task failed because of too few users_1.')
@@ -591,7 +583,7 @@ class SecureFedAvgScheduler(FedAvgScheduler):
         torch.save(masked_result, buffer)
         self.push_log('Pushing local update to the aggregator ...')
         self.data_channel.send_stream(source=self.id,
-                                      target=[self._aggregator],
+                                      target=self._aggregator,
                                       data_stream=buffer.getvalue())
         self.push_log('Successfully pushed local update to the aggregator.')
 
